import click
import IPython
import os
import yaml

@click.group()
def admincli():
    pass

@admincli.command()
@click.option('--output', '-o')
def mk_config(output):
    config_data = {}
    for k,v in os.environ.iteritems():
        if k.startswith('RECAST_'):
            config_data[k.replace('RECAST_','')] = v
    with open(output, 'w') as outputfile:
        outputfile.write(yaml.dump(config_data,default_flow_style=False))



import recastdb.models
from recastdb.database import db

from eve import Eve
from eve_sqlalchemy import SQL
from eve_sqlalchemy.validation import ValidatorSQL
from eve_swagger import swagger

import uuid
import werkzeug.datastructures

from settings import DOMAIN, SQLALCHEMY_DATABASE_URI, DEBUG, PAGINATION_DEFAULT, EMBEDDING
from settings import XML, JSON, RESOURCE_METHODS, PUBLIC_METHODS, ITEM_METHODS
from settings import PUBLIC_ITEM_METHODS, HATEOAS, IF_MATCH, ID_FIELD, ITEM_LOOKUP_FIELD
from settings import AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY, AWS_S3_BUCKET_NAME
from settings import TokenAuth

from boto3.session import Session
import requests as httprequest
import json

SETTINGS = {
	'PAGINATION_LIMIT': PAGINATION_DEFAULT,
	'PAGINATION_DEFAULT':PAGINATION_DEFAULT,
	'DOMAIN': DOMAIN,
	'SQLALCHEMY_DATABASE_URI': SQLALCHEMY_DATABASE_URI,
	'DEBUG': DEBUG,
	'XML': XML,
	'JSON': JSON,
	'RESOURCE_METHODS': RESOURCE_METHODS,
	'ITEM_METHODS': ITEM_METHODS,
	'PUBLIC_METHODS': PUBLIC_METHODS,
	'PUBLIC_ITEM_METHODS': PUBLIC_ITEM_METHODS,
	'HATEOAS': HATEOAS,
	'IF_MATCH': IF_MATCH,
	'ID_FIELD': ID_FIELD,
	'ITEM_LOOKUP_FIELD': ITEM_LOOKUP_FIELD,
	'SQLALCHEMY_TRACK_MODIFICATIONS': True,
	'EMBEDDING': EMBEDDING
}



def pre_request_archives_post_callback(request, lookup=None):
	print type(request.files),request.files
	print type(request.data),request.data
	print type(request.form),request.form
	print type(request.json),request.json
	print dir(request)

	upload_with_tag(request,'request')

def pre_response_archives_post_callback(request, lookup=None):
	upload_with_tag(request,'response')

def upload_with_tag(request,tag):
	"""
		Uploads File file to AWS s3 with tag (either request or response)
	"""

	zip_file = request.files.get('file')
	request.files = werkzeug.datastructures.ImmutableMultiDict({})
	orcid_id = request.__dict__['authorization']['username']
	if zip_file:
		aws_filename = str(uuid.uuid4())
		modified_form = request.form.to_dict()
		modified_form.update(file_name = aws_filename)
		request.form = werkzeug.datastructures.ImmutableMultiDict(
			modified_form
		)

		metadata = {'tag': tag,
					'username': orcid_id,
					'originalname': zip_file.filename,
					}
		upload_AWS(zip_file, aws_filename, metadata)



def upload_AWS(zip_file, file_uuid, metadata=None):
	session = Session(AWS_ACCESS_KEY_ID,AWS_SECRET_ACCESS_KEY)
	s3 = session.resource('s3')
	s3.Bucket(AWS_S3_BUCKET_NAME).put_object(
		Key=str(file_uuid),
		Body=zip_file,
		ACL='public-read',
		Metadata=metadata)

def download_AWS(file_name, original_file_name, download_path=None):
	session = Session(AWS_ACCESS_KEY_ID,AWS_SECRET_ACCESS_KEY)
	s3 = session.resource('s3')
	out_file = download_path or original_file_name
	s3.Bucket(AWS_S3_BUCKET_NAME).download_file(file_name, out_file)

def after_fetch_archives(request, lookup=None):
	""" Function to add download link in the response """
	url = 'https://s3.amazonaws.com/{}/{}'.format(AWS_S3_BUCKET_NAME, request['file_name'])
	r = httprequest.get(url)
	if r.ok:
		request['file_link'] = url
	else:
		request['file_link'] = ''

def request_archive_patch_callback(request, lookup = None):
	upload_with_tag(request,'response')

def response_archive_patch_callback(request, lookup = None):
	upload_with_tag(request,'response')

app = Eve(auth=TokenAuth, settings=SETTINGS, validator=ValidatorSQL, data=SQL)


app.register_blueprint(swagger)
app.config['SWAGGER_INFO'] = {
    'title': 'RECAST API',
    'version': '1.0',
    'description': 'RESTful API to RECAST Service',
    'termsOfService': 'use at your own risk',
    'contact': {
        'name': 'Lukas Heinrich',
        'url': 'https://recast-frontend.cern.ch'
    },
    'license': {
        'name': 'BSD',
        'url': 'https://recast-frontend.cern.ch',
    }
}


app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = True

app.on_pre_POST_request_archives      += pre_request_archives_post_callback
app.on_pre_PATCH_response_archives 	  += request_archive_patch_callback
app.on_fetched_item_request_archives  += after_fetch_archives

app.on_pre_POST_response_archives 	  += pre_response_archives_post_callback
app.on_pre_PATCH_response_archives 	  += response_archive_patch_callback
app.on_fetched_item_response_archives += after_fetch_archives

Base = recastdb.database.db.Model

#bind SQLAlchemy
db = app.data.driver
Base.metadata.bind = db.engine
db.Model = Base()



from recastrestapi.apiconfig import config as apiconf

import recastdb.models
from recastdb.database import db

from eve_sqlalchemy.decorators import registerSchema
from eve.utils import config
from recastrestapi.apiconfig import config as apiconf

DEBUG = True
SQLALCHEMY_DATABASE_URI =  apiconf['RECAST_DBPATH']
AWS_ACCESS_KEY_ID = apiconf['RECAST_AWS_ACCESS_KEY_ID']
AWS_SECRET_ACCESS_KEY = apiconf['RECAST_AWS_SECRET_ACCESS_KEY']
AWS_S3_BUCKET_NAME = apiconf['RECAST_AWS_S3_BUCKET_NAME']
RESOURCE_METHODS = ['GET', 'POST', 'DELETE']
ITEM_METHODS = ['GET', 'PUT', 'DELETE']
PUBLIC_METHODS = ['GET']
PUBLIC_ITEM_METHODS = ['GET']
HATEOAS = True
IF_MATCH = False
PAGINATION_DEFAULT = 1000
ID_FIELD = 'id'
ITEM_LOOKUP_FIELD = ID_FIELD
config.ID_FIELD = ID_FIELD
config.ITEM_LOOKUP_FIELD = ID_FIELD
XML = True
JSON = True
EMBEDDING = True

from eve.auth import BasicAuth
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.orm.exc import MultipleResultsFound
class TokenAuth(BasicAuth):
	def check_auth(self, orcid_id, token, allowed_roles, resource, method):
		"""
			Token based authentications
		"""
		try:
			user = recastdb.models.User.query.filter(
				recastdb.models.User.orcid_id == orcid_id).one()
			access_token = recastdb.models.AccessToken.query.filter(
				recastdb.models.AccessToken.token == token).one()
			return access_token.user_id == user.id
		except MultipleResultsFound, e:
			return False
		except NoResultFound, e:
			return False

registerSchema('users')(recastdb.models.User)
registerSchema('analysis')(recastdb.models.Analysis)
registerSchema('subscriptions')(recastdb.models.Subscription)
registerSchema('run_conditions')(recastdb.models.RunCondition)
registerSchema('models')(recastdb.models.Model)
registerSchema('scan_requests')(recastdb.models.ScanRequest)
registerSchema('point_requests')(recastdb.models.PointRequest)
registerSchema('basic_requests')(recastdb.models.BasicRequest)
registerSchema('parameters')(recastdb.models.Parameters)
registerSchema('point_coordinates')(recastdb.models.PointCoordinate)
registerSchema('request_archives')(recastdb.models.RequestArchive)
registerSchema('scan_responses')(recastdb.models.ScanResponse)
registerSchema('point_responses')(recastdb.models.PointResponse)
registerSchema('basic_responses')(recastdb.models.BasicResponse)
registerSchema('response_archives')(recastdb.models.ResponseArchive)

DOMAIN = {
    'users': recastdb.models.User._eve_schema['users'],
    'analysis': recastdb.models.Analysis._eve_schema['analysis'],
    'subscriptions': recastdb.models.Subscription._eve_schema['subscriptions'],
    'run_conditions': recastdb.models.RunCondition._eve_schema['run_conditions'],
    'models': recastdb.models.Model._eve_schema['models'],
    'scan_requests': recastdb.models.ScanRequest._eve_schema['scan_requests'],
    'point_requests': recastdb.models.PointRequest._eve_schema['point_requests'],
    'basic_requests': recastdb.models.BasicRequest._eve_schema['basic_requests'],
    'parameters': recastdb.models.Parameters._eve_schema['parameters'],
    'point_coordinates': recastdb.models.PointCoordinate._eve_schema['point_coordinates'],
    'request_archives': recastdb.models.RequestArchive._eve_schema['request_archives'],
    'scan_responses': recastdb.models.ScanResponse._eve_schema['scan_responses'],
    'point_responses': recastdb.models.PointResponse._eve_schema['point_responses'],
    'basic_responses': recastdb.models.BasicResponse._eve_schema['basic_responses'],
    'response_archives': recastdb.models.ResponseArchive._eve_schema['response_archives'],
}

DOMAIN['users'].update({
    'item_lookup_field': 'id',
    'additional_lookup': {
        'url': 'regex("[0-9]+")',
        'field': 'id'
    },
    'cache_control': 'max-age=10,must-revalidate',
    'cache_expires': 10,
    'resource_methods': ['GET', 'POST', 'DELETE'],
    'item_methods': ['GET', 'PUT', 'PATCH']
})
## prevent email addresses from being returned
DOMAIN['users']['datasource']['projection']['email'] = 0

DOMAIN['analysis'].update({
    'item_lookup_field': 'id',
    'cache_control': 'max-age=10,must-revalidate',
    'cache_expires': 10,
    'resource_methods': ['GET', 'POST', 'DELETE']
})

DOMAIN['run_conditions'].update({
    'item_lookup_field': 'id',
    'additional_lookup': {
        'url': 'regex("[0-9]+")',
        'field': 'id'
    },
    'cache_control': 'max-age=10,must-revalidate',
    'cache_expires': 10,
    'resource_methods': ['GET', 'POST', 'DELETE'],
    'item_methods': ['GET', 'PUT', 'PATCH'],
    'resource_methods': ['GET', 'POST', 'DELETE']
})

DOMAIN['request_archives'].update({
    'allow_unknown': True,
    'resource_methods': ['GET', 'POST', 'DELETE'],
    'item_methods': ['GET', 'PUT', 'PATCH'],
})

DOMAIN['response_archives'].update({
    'allow_unknown': True,
    'resource_methods': ['GET', 'POST', 'DELETE'],
    'item_methods': ['GET', 'PUT', 'PATCH'],
})

DOMAIN['scan_requests'].update({
    'item_lookup_field': 'id',
    'additional_lookup': {
        'url': 'regex("[0-9]+")',
        'field': 'id'
    },
    'cache_control': 'max-age=10,must-revalidate',
    'cache_expires': 10,
    'resource_methods': ['GET', 'POST', 'DELETE'],
    'item_methods': ['GET', 'PUT', 'PATCH'],
    'url': 'scan_requests',
})

DOMAIN['scan_responses'].update({
    'item_lookup_field': 'id',
    'additional_lookup': {
        'url': 'regex("[0-9]+")',
        'field': 'id'
    },
    'cache_control': 'max-age=10,must-revalidate',
    'cache_expires': 10,
    'resource_methods': ['GET', 'POST', 'DELETE'],
    'item_methods': ['GET', 'PUT', 'PATCH'],
    'url': 'scan_responses',
})

DOMAIN['point_requests'].update({
    'item_lookup_field': 'id',
    'additional_lookup': {
        'url': 'regex("[0-9]+")',
        'field': 'id'
    },
    'cache_control': 'max-age=10,must-revalidate',
    'cache_expires': 10,
    'resource_methods': ['GET', 'POST', 'DELETE'],
    'item_methods': ['GET', 'PUT', 'PATCH'],
    'allow_unknown': True,
})

DOMAIN['point_responses'].update({
    'item_lookup_field': 'id',
    'additional_lookup': {
        'url': 'regex("[0-9]+")',
        'field': 'id'
    },
    'cache_control': 'max-age=10,must-revalidate',
    'cache_expires': 10,
    'resource_methods': ['GET', 'POST', 'DELETE'],
    'item_methods': ['GET', 'PUT', 'PATCH'],
    'url': 'point_responses',
})

DOMAIN['basic_requests'].update({
    'item_lookup_field': 'id',
    'additional_lookup': {
        'url': 'regex("[0-9]+")',
        'field': 'id'
    },
    'cache_control': 'max-age=10,must-revalidate',
    'cache_expires': 10,
    'resource_methods': ['GET', 'POST', 'DELETE'],
    'item_methods': ['GET', 'PUT', 'PATCH'],
    'allow_unknown': True,
})

DOMAIN['basic_responses'].update({
    'item_lookup_field': 'id',
    'additional_lookup': {
        'url': 'regex("[0-9]+")',
        'field': 'id'
    },
    'cache_control': 'max-age=10,must-revalidate',
    'cache_expires': 10,
    'resource_methods': ['GET', 'POST', 'DELETE'],
    'item_methods': ['GET', 'PUT', 'PATCH'],
    'url': 'basic_responses',
})


# import IPython
# IPython.embed()

DOMAIN['point_coordinates'].update({
    'item_lookup_field': 'id',
    'additional_lookup': {
        'url': 'regex("[0-9]+")',
        'field': 'id'
    },
    'cache_control': 'max-age=10,must-revalidate',
    'cache_expires': 10,
    'resource_methods': ['GET', 'POST', 'DELETE'],
    'item_methods': ['GET', 'PUT', 'PATCH'],
    'url': 'point_coordinates',
})



import unittest
import json
from recastrestapi.server import app
from recastrestapi.server import db
from recastdb.models import AccessToken, User
import random
import string

from flask import url_for

class APITestCase(unittest.TestCase):
    def setUp(self):
        self.app = app
        self.app.testing = True
        self.app.app_context().push()
        db.create_all()
        self.client = self.app.test_client()
        self.token = self.random_string()
        
    def tearDown(self):
        db.session.remove()

    def test_show_user(self):
        user = User(name="john alex", email="johnalex@example.com")
        db.session.add(user)
        db.session.commit()
        
        response = self.client.get('/users/'+str(user.id), headers=self.get_api_header(self.token))
        self.assertTrue(response.status_code == 200)
        
    def test_analyses(self):
        """Analyses"""
        response = self.client.get('/analyses/1', headers=self.get_api_header(self.token))
        self.assertTrue(response.status_code == 200)

    def test_home(self):
        """Page not requiring TOKEN(not implemented)"""
        pass
        
    def test_users(self):
        user = User.query.first()
        self.assertIsNotNone(user)
        user1 = User(name='john', email='john@example.com')
        user2 = User(name='Alex', email='alex@example.com')
        db.session.add(user1)
        db.session.add(user2)
        db.session.commit()

        response = self.client.get('/users/1', headers=self.get_api_header(self.token))
        self.assertTrue(response.status_code == 200)

    def test_POST_USER(self):
        """Add user through POST(Not implemented)"""
        user = {'name': 'Matt',
                'email': 'matt@example.com'
                }
        
        response = self.client.get('/users', headers=self.get_api_header(self.token))
        print response.status_code

    def test_add_token(self):
        """Adding Random Token"""

    def tearDown(self):
        db.session.remove()
        #self.app.app_context().pop()

    def test_add_token(self):
        test_user = User(name="Rest API test user", email="testapi@email.com")
        db.session.add(test_user)
        db.session.commit()
        token = AccessToken(token=self.token, user_id=test_user.id)
        db.session.add(token)
        db.session.commit()

    def get_api_header(self, token="test_token"):
        return {
            'Authorization': 'Basic ' + token, 
            'Accept': 'application/json',
            'Content-Type': 'application/json'
            }

    def test_404(self):
        """Wrong URL"""
        response = self.client.get(
            '/wrong/url',
            headers=self.get_api_header())
        self.assertTrue(response.status_code == 404)


    def test_401(self):
         """Unauthorized access"""
         response = self.client.get(
             '/users',
             headers=self.get_api_header("invalid_token"))
         self.assertTrue(response.status_code == 401)

    def test_token_auth(self):
        """(Not implemented)"""
        pass

    def test_GET(self):
        """(Not implemented)"""
        pass

    def test_POST(self):
        """(Not implemented)"""
        json_response = json.loads(response.data.decode('utf-8'))
        self.assertEquals(json_response['_error']['code'],404)

    def test_token_auth(self):
        pass

    def test_GET(self):
        pass

    def test_POST(self):
        pass

    def random_string(self, size=10, chars=string.ascii_uppercase + string.digits):
        return ''.join(random.choice(chars) for _ in range(size))
        



import os
import pkg_resources
import yaml
import logging
log = logging.getLogger(__name__)
def default_config():
  return yaml.load(open(pkg_resources.resource_filename('recastrestapi','resources/defaultconfig.yaml')))

def mk_config():
    the_config = default_config()
    if os.environ.has_key('RECASTAPI_CONFIG'):
        custom_config = yaml.load(open(os.environ['RECASTAPI_CONFIG']))
        the_config.update(**custom_config)
    for k,v in os.environ.iteritems():
        if k.startswith('RECAST_'):
            the_config[k] = v
    #replicate RECAST_* back to shell env
    for k,v in the_config.iteritems():
        log.info('config value %s => %s',k,v)
        if not k.startswith('RECAST_'):
            raise ValueError('configuration keys must starte with RECAST_!. change key: {}'.format(k))
        os.environ[k] = str(v)
    return the_config

config = mk_config()



from recastrestapi.apiconfig import config as apiconf

import recastdb.models
from recastdb.database import db

from eve_sqlalchemy.decorators import registerSchema
from eve.utils import config
from recastrestapi.apiconfig import config as apiconf

DEBUG = True
SQLALCHEMY_DATABASE_URI =  apiconf['RECAST_DBPATH']
AWS_ACCESS_KEY_ID = apiconf['RECAST_AWS_ACCESS_KEY_ID']
AWS_SECRET_ACCESS_KEY = apiconf['RECAST_AWS_SECRET_ACCESS_KEY']
AWS_S3_BUCKET_NAME = apiconf['RECAST_AWS_S3_BUCKET_NAME']
RESOURCE_METHODS = ['GET', 'POST', 'DELETE']
PUBLIC_METHODS = ['GET']
PUBLIC_ITEM_METHODS = ['GET']
HATEOAS = True
IF_MATCH = False
PAGINATION_DEFAULT = 1000

ID_FIELD = 'id'
ITEM_LOOKUP_FIELD = ID_FIELD
config.ID_FIELD = ID_FIELD
config.ITEM_LOOKUP_FIELD = ID_FIELD
XML = False
JSON = True

registerSchema('users')(recastdb.models.User)

DOMAIN = {
    'users': recastdb.models.User._eve_schema['users'],
}



from setuptools import setup, find_packages

setup(
    name = 'recast-rest-api',
    version = '0.1.0',
    description = 'API for the RECAST project',
    url = 'https://github.com/cbora/recast-rest-api',
    author = 'Lukas Heinrich, Christian Bora',
    author_email = 'lukas.heinrich@cern.ch, borachristian@gmail.com',
    packages = find_packages(),
    include_package_data = True,
    install_requires = [
        'Flask==0.10.1',
        'werkzeug==0.10.4',
#        'Eve==0.5.3',
        'eve-sqlalchemy',
        'markupsafe==0.23',
        'celery',
        'pyyaml',
        'psycopg2',
        'click',
        'boto3',
        'requests',
        'recast-database',
        'eve-swagger'
        ],
    entry_points = {
        'console_scripts': [
            'recast-api = recastrestapi.apicli:apicli',
            'recast-api-admin = recastrestapi.admincli:admincli',
            ]
        },
    dependency_links = [
        'https://github.com/recast-hep/recast-database/tarball/master#egg=recast-database-0.1.0',
        ]
    )
    



import click
import os
import subprocess
import logging
log = logging.basicConfig(level = logging.INFO)
def set_config(config):
    if config:
        os.environ['RECASTAPI_CONFIG'] = config
    from apiconfig import config as apiconfig


@click.group()
def apicli():
    pass

@apicli.command()
@click.option('--config', '-c')
def server(config):
    set_config(config)
    from server import app
    port = int(os.environ.get("RECAST_PORT", 5000))
    app.run(host='0.0.0.0', port=port, ssl_context = (os.environ['RECAST_SSL_CERT'],os.environ['RECAST_SSL_KEY']) )

@apicli.command()
@click.option('--config', '-c')
def test(config):
    set_config(config)
    import unittest
    tests = unittest.TestLoader().discover('tests')
    unittest.TextTestRunner(verbosity=2).run(tests)
